@@ -7,7 +7,7 @@ package IO::Async::ChildManager;
 
 use strict;
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 # Not a notifier
 
@@ -274,6 +274,12 @@ will be invoked in the following way:
 This key is optional; if not supplied, the calling code should install a
 handler using the C<watch_child()> method.
 
+=item keep_signals => BOOL
+
+Optional boolean. If missing or false, any CODE references in the C<%SIG> hash
+will be removed and restored back to C<DEFAULT> in the child process. If true,
+no adjustment of the C<%SIG> hash will be performed.
+
 =cut
 
 sub detach_child
@@ -287,6 +293,13 @@ sub detach_child
    defined $kid or croak "Cannot fork() - $!";
 
    if( $kid == 0 ) {
+      unless( $params{keep_signals} ) {
+         foreach( keys %SIG ) {
+            next if m/^__(WARN|DIE)__$/;
+            $SIG{$_} = "DEFAULT" if ref $SIG{$_} eq "CODE";
+         }
+      }
+
       my $exitvalue = eval { $code->() };
 
       defined $exitvalue or $exitvalue = -1;
@@ -530,7 +543,7 @@ sub _spawn_in_parent
    $set->add( IO::Async::Buffer->new(
       read_handle => $readpipe,
 
-      on_incoming_data => sub {
+      on_read => sub {
          my ( $self, $buffref, $closed ) = @_;
 
          if( !defined $dollarbang ) {
@@ -588,19 +601,43 @@ sub _spawn_in_child
    my $exitvalue = eval {
       my %keep_fds = ( 0 => 1, 1 => 1, 2 => 1 ); # Keep STDIN, STDOUT, STDERR
 
+      my $max_fd = 0;
+      my $writepipe_clashes = 0;
+
       if( @$setup ) {
          # The writepipe might be in the way of a setup filedescriptor. If it
          # is we'll have to dup2() it out of the way then close the original.
-         my ( $max_fd, $writepipe_clashes ) = ( 0, 0 );
          foreach my $i ( 0 .. $#$setup/2 ) {
-            my $key = $$setup[$i*2];
+            my ( $key, $value ) = @$setup[$i*2, $i*2 + 1];
             $key =~ m/^fd(\d+)$/ or next;
             my $fd = $1;
 
             $max_fd = $fd if $fd > $max_fd;
             $writepipe_clashes = 1 if $fd == fileno $writepipe;
-         }
 
+            my ( $operation, @params ) = @$value;
+
+            $operation eq "close" and do {
+               delete $keep_fds{$fd};
+            };
+
+            $operation eq "dup" and do {
+               my $fileno = fileno $params[0];
+               # Keep a count of how many times it will be dup()ed from so we
+               # can close it once we've finished
+               $keep_fds{$fileno}++;
+            };
+         }
+      }
+
+      $keep_fds{fileno $writepipe} = 1;
+
+      foreach ( 0 .. OPEN_MAX_FD ) {
+         next if exists $keep_fds{$_};
+         POSIX::close( $_ );
+      }
+
+      if( @$setup ) {
          if( $writepipe_clashes ) {
             $max_fd++;
 
@@ -616,17 +653,20 @@ sub _spawn_in_child
                my $fd = $1;
                my( $operation, @params ) = @$value;
 
-               $operation eq "close" and do {
-                  delete $keep_fds{$fd};
-                  next;
-               };
-
-               $keep_fds{$fd} = 1;
-
                $operation eq "dup"   and do {
                   my $from = fileno $params[0];
-                  dup2( $from, $fd ) or die "Cannot dup2($from to $fd) - $!\n";
+
+                  if( $from != $fd ) {
+                     POSIX::close( $fd );
+                     dup2( $from, $fd ) or die "Cannot dup2($from to $fd) - $!\n";
+                  }
+
+                  $keep_fds{$from}--;
+                  if( !$keep_fds{$from} ) {
+                     POSIX::close( $from );
+                  }
                };
+
                $operation eq "open"  and do {
                   my ( $mode, $filename ) = @params;
                   open( my $fh, $mode, $filename ) or die "Cannot open('$mode', '$filename') - $!\n";
@@ -643,13 +683,6 @@ sub _spawn_in_child
          }
       }
 
-      $keep_fds{fileno $writepipe} = 1;
-
-      foreach ( 0 .. OPEN_MAX_FD ) {
-         next if exists $keep_fds{$_};
-         POSIX::close( $_ );
-      }
-
       $code->();
    };
 
