@@ -7,7 +7,7 @@ package IO::Async::Set;
 
 use strict;
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 use Carp;
 
@@ -95,13 +95,9 @@ sub _nkey
    my $self = shift;
    my ( $notifier ) = @_;
 
-   # We key the notifiers by their reading fileno; because every notifier
-   # needs to have one.
-   my $nkey = $notifier->read_fileno;
-
-   defined $nkey or croak "Cannot operate on a notifer that is not read-bound to a handle with a fileno";
-
-   return $nkey;
+   # References in integer context yield their address. We'll use that as the
+   # notifier key
+   return $notifier + 0;
 }
 
 =head2 $set->add( $notifier )
